#!/usr/bin/python
""" Cog content generation tool.
    http://nedbatchelder.com/code/cog

    Copyright 2004-2019, Ned Batchelder.
"""

import sys
from cogapp import Cog

sys.exit(Cog().main(sys.argv))
